@@ -17,8 +17,10 @@ import json
 import logging
 import random
 import shutil
+import sys
 import time
 import uuid
+import warnings
 import zipfile
 
 # for type checking
@@ -48,7 +50,7 @@ _IDENTITY = np.eye(4, dtype=np.float64)
 _IDENTITY.flags["WRITEABLE"] = False
 
 
-def has_module(name):
+def has_module(name: str) -> bool:
     """
     Check to see if a module is installed by name without
     actually importing the module.
@@ -63,28 +65,14 @@ def has_module(name):
     installed : bool
       True if module is installed
     """
-    # this should work on Python 2.7 and 3.4+
-    import pkgutil
+    if sys.version_info >= (3, 10):
+        # pkgutil was deprecated
+        from importlib.util import find_spec
+    else:
+        # this should work on Python 2.7 and 3.4+
+        from pkgutil import find_loader as find_spec
 
-    return pkgutil.find_loader(name) is not None
-
-
-try:
-    import rtree
-
-    # some versions of rtree screw up indexes on stream loading
-    # do a test here so we know if we are free to use stream loading
-    assert (
-        next(
-            rtree.index.Index(
-                [(1564, [0, 0, 0, 10, 10, 10], None)],
-                properties=rtree.index.Property(dimension=3),
-            ).intersection([1, 1, 1, 2, 2, 2])
-        )
-        == 1564
-    )
-except BaseException as E:
-    rtree = E
+    return find_spec(name) is not None
 
 
 def unitize(vectors, check_valid=False, threshold=None):
@@ -142,22 +130,18 @@ def unitize(vectors, check_valid=False, threshold=None):
     return unit
 
 
-def euclidean(a, b):
+def euclidean(a, b) -> float:
     """
-    Euclidean distance between vectors a and b.
-
-    Parameters
-    ------------
-    a : (n,) float
-       First vector
-    b : (n,) float
-       Second vector
-
-    Returns
-    ------------
-    distance : float
-        Euclidean distance between A and B
+    DEPRECATED: use `np.linalg.norm(a - b)` instead of this.
     """
+    warnings.warn(
+        "`trimesh.util.euclidean` is deprecated "
+        + "and will be removed in January 2025. "
+        + "replace with `np.linalg.norm(a - b)`",
+        category=DeprecationWarning,
+        stacklevel=2,
+    )
+
     a = np.asanyarray(a, dtype=np.float64)
     b = np.asanyarray(b, dtype=np.float64)
     return np.sqrt(((a - b) ** 2).sum())
@@ -199,7 +183,7 @@ def is_pathlib(obj):
     return hasattr(obj, "absolute") and name.endswith("Path")
 
 
-def is_string(obj):
+def is_string(obj) -> bool:
     """
     Check if an object is a string.
 
@@ -216,7 +200,7 @@ def is_string(obj):
     return isinstance(obj, str)
 
 
-def is_none(obj):
+def is_none(obj) -> bool:
     """
     Check to see if an object is None or not.
 
@@ -239,7 +223,7 @@ def is_none(obj):
     return False
 
 
-def is_sequence(obj):
+def is_sequence(obj) -> bool:
     """
     Check if an object is a sequence or not.
 
@@ -1765,6 +1749,8 @@ def bounds_tree(bounds):
     tree : Rtree
       Tree containing bounds by index
     """
+    import rtree
+
     # make sure we've copied bounds
     bounds = np.array(bounds, dtype=np.float64, copy=True)
     if len(bounds.shape) == 3:
@@ -2198,7 +2184,7 @@ def generate_basis(z, epsilon=1e-12):
     return result
 
 
-def isclose(a, b, atol):
+def isclose(a, b, atol: float = 1e-8):
     """
     A replacement for np.isclose that does fewer checks
     and validation and as a result is roughly 4x faster.
@@ -2221,12 +2207,10 @@ def isclose(a, b, atol):
       Per-element closeness
     """
     diff = a - b
-    close = np.logical_and(diff > -atol, diff < atol)
-
-    return close
+    return np.logical_and(diff > -atol, diff < atol)
 
 
-def allclose(a, b, atol=1e-8):
+def allclose(a, b, atol: float = 1e-8):
     """
     A replacement for np.allclose that does few checks
     and validation and as a result is faster.
@@ -2244,6 +2228,7 @@ def allclose(a, b, atol=1e-8):
     -----------
     bool indicating if all elements are within `atol`.
     """
+    #
     return float((a - b).ptp()) < atol
 
 
